<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLIterator;

/**
 * ImageGalleryParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Parsers
 * @since   Infomaker\Everyware\NewsML\Parsers\ImageGalleryParser 1.0.0
 */
class ImageGalleryParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/imagegallery';

    /**
     * @var LinkParser
     */
    protected $linkParser;

    public function __construct()
    {
        $this->linkParser = new LinkParser([
            ImageLinkParser::LINK_TYPE => new ImageLinkParser()
        ]);
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new Item($element);
        $item->fill([
            'type' => $item->getType(),
            'images' => Arr::flatten($this->linkParser->parseLinks($element), 1)
        ]);

        return [$item];
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }
}
