<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use SimpleXMLIterator;

/**
 * ImageParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Parsers
 * @since   Infomaker\Everyware\NewsML\Parsers\ImageParser 1.0.0
 */
class ImageParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/image';

    /**
     * @var LinkParser
     */
    protected $linkParser;

    /**
     * ObjectParser constructor.
     *
     * @since 0.1
     *
     * @param ItemParser[]
     */
    public function __construct()
    {
        $this->linkParser = new LinkParser([
            ImageLinkParser::LINK_TYPE => new ImageLinkParser()
        ]);
    }

    /**
     * @param SimpleXMLIterator $element
     *
     * @since 1.0.0
     * @return mixed
     */
    public function parse(SimpleXMLIterator $element): array
    {
        return Arr::flatten($this->linkParser->parseLinks($element), 1);
    }

    /**
     * Retrieve the type to pars
     *
     * @since 1.0.0
     * @return string
     */
    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }
}
