<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLElement;
use SimpleXMLIterator;

class LinkParser
{

    /**
     * Contains a group of parsers
     *
     * @var ItemParser[]
     */
    private $parsers;

    /**
     * LinkParser constructor.
     *
     * @since 0.1
     *
     * @param ItemParser[]
     */
    public function __construct(array $parsers = [])
    {
        $this->parsers = $parsers;
    }

    public function getType(SimpleXMLElement $el): string
    {
        return Arr::get(Arr::get((array)$el->attributes(), '@attributes', []), 'type', '');
    }

    public function getLinkParser(SimpleXMLElement $el): ?ItemParser
    {  
        return Arr::get($this->parsers, $this->getType($el), null);
    }

    /**
     * @param SimpleXMLIterator $el
     *
     * @since 1.0.0
     * @return Item[]
     */
    public function parseLinks(SimpleXMLIterator $el): array
    {
        $links = [];
        if (isset($el->links->link)) {
            foreach ($el->links->link as $tag => $item) {
                $parser = $this->getLinkParser($item);
                $links[] = $parser instanceof ItemParser ? $parser->parse($item) : new Item($item);
            }
        }
        return Arr::flatten($links, 1);
    }
}
