<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLIterator;

class YoutubeParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/youtube';

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new Item($element);
        $item->fill([
            'type' => $item->getType(),
            'uri' => $this->createUri((string)$item->getItemAttribute('uri', '')),
            'thumb' => $this->getThumbData($element)
        ]);

        return [$item];
    }

    private function getThumbData(SimpleXMLIterator $element): array
    {
        $links = Arr::flatten($this->parseLinks($element), 1);
        foreach ($links as $link) {
            if ($link instanceof Item && $link->getType() === 'image/jpg') {
                return $this->parseThumb($link);
            }
        }

        return [];
    }
    private function parseLinks(SimpleXMLIterator $el): array
    {
        $links = [];
        if (isset($el->links->link)) {
            foreach ($el->links->link as $tag => $item) {
                $links[] = new Item($item);
            }
        }
        return $links;
    }

    private function createUri(string $uri): string
    {
        return str_replace('watch?v=', 'embed/', $uri);
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }

    private function parseThumb(Item $link): array
    {
        return array_replace([
            'width' => 0,
            'height' => 0,
            'url' => $link->getItemAttribute('url', '')
        ], (array)($link->getChild('data') ?? []));
    }
}
