<?php

namespace Spec\Infomaker\Everyware\NewsML;

use Infomaker\Everyware\NewsML\Item;
use PhpSpec\Exception\Example\FailureException;
use PhpSpec\ObjectBehavior;
use SimpleXMLIterator;
require_once(__DIR__ . '/xmlStr.php');

class ItemSpec extends ObjectBehavior
{
    private $realisticXML, $simplerXML, $attributes;

    public function let()
    {
        $this->realisticXML = realisticXML;
        $this->simplerXML = simplerXML;
        $this->attributes = attributes;
        $el = simplexml_load_string($this->realisticXML, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
    }
    public function it_is_initializable()
    {
        $this->shouldHaveType(Item::class);
    }
    public function it_should_filter_inline_content()
    {
        $text = '<p>Test paragraph.</p><!-- Comment --> <a href="#fragment">Other text</a>';

        $this->filterInlineContent($text, [])
            ->shouldReturn('<p>Test paragraph.</p><!-- Comment --> <a href="#fragment">Other text</a>');
        $this->filterInlineContent($text, ['a'])
            ->shouldReturn('Test paragraph. <a href="#fragment">Other text</a>');
    }
    public function it_should_get_item_attribute()
    {
        $xml = '
            <xml stuff="it works!">
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getItemAttribute('stuff')
            ->shouldReturn('it works!');
    }
    public function it_should_return_a_default_value_if_item_attribute_is_not_found()
    {
        $this->getItemAttribute('notFound', 'defaultValue')
            ->shouldReturn('defaultValue');
    }
    public function it_should_return_a_child()
    {
        $this->getChild('links')
            ->shouldHaveArrayKey('link');
    }
    public function it_should_get_type()
    {
        $xml = '
            <xml type="x-im/content-part">
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getType()
            ->shouldReturn('x-im/content-part');
    }
    public function it_should_remove_all_line_breaks()
    {
        $this->removeLinebreak($this->simplerXML)
            ->shouldReturn('<movies><movie>abcd</movie><movie>efgh</movie><movie>hijk</movie></movies>');
    }
    public function it_should_get_content_as_xml()
    {
        $el = new SimpleXmlIterator($this->simplerXML, null);
        $this->beConstructedWith($el);
        $this->getContentAsXML()
            ->shouldReturn('<?xml version="1.0"?><movies><movie>abcd</movie><movie>efgh</movie><movie>hijk</movie></movies>');
    }
    public function it_should_get_filtered_content()
    {
        $this->getFilteredContent(['attributes', 'name', 'game', 'type', 'title', 'data'])
            ->shouldReturn('<attributes id="MjQ0LDI4LDIxNSwxNzk" title="NEW Fancy teaser with fancy article with fancy stuff" type="x-im/teaser"/><attributes name="one" game="lonely"><data>35202347</data></attributes>');
    }
    public function getMatchers(): array
    {
        return [
            'haveKeys' => function ($subject, $keys) {
                if (count(array_intersect_key(array_flip($keys), $subject)) === count($keys)) {
                    return true;
                }
            },
            'haveValue' => function (array $subject, string $key, $value) {
                return array_filter($subject, function($item) use($value, $key){
                    return ($item->$key === $value);
                 });
            },
            'haveArrayKey' => function ($subject, $key) {
                if (!array_key_exists($key, $subject)) {
                    throw new FailureException(sprintf(
                        'Message with subject "%s" and key "%s".',
                        array_shift($subject),
                        $key
                    ));
                }
                return true;
            },
        ];
    }
}
