<?php

namespace Spec\Infomaker\Everyware\NewsML;

use Infomaker\Everyware\NewsML\NewsMLTransformer;
use Infomaker\Everyware\NewsML\Parsers\IdfParser;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use Infomaker\Everyware\NewsML\Parsers\ImageGalleryParser;
use Infomaker\Everyware\NewsML\Parsers\ObjectParser;
use Infomaker\Everyware\NewsML\Parsers\ImageParser;
use Infomaker\Everyware\NewsML\Parsers\ContentPartParser;
use Infomaker\Everyware\NewsML\Parsers\GroupParser;
use Infomaker\Everyware\NewsML\Parsers\TeaserParser;
use Infomaker\Everyware\NewsML\Parsers\LinkObjectParser;
use Infomaker\Everyware\NewsML\Parsers\ReviewParser;
use Infomaker\Everyware\NewsML\Parsers\PdfParser;
use Infomaker\Everyware\NewsML\Parsers\YoutubeParser;
use Infomaker\Everyware\NewsML\Parsers\HtmlEmbedParser;
use Infomaker\Everyware\NewsML\Parsers\TableParser;
use Infomaker\Everyware\NewsML\Parsers\SocialEmbedParser;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class NewsMLTransformerSpec extends ObjectBehavior
{
    private $parsers;

    function let()
    {
        $this->parsers = [
            SocialEmbedParser::OBJECT_TYPE => new SocialEmbedParser(),
            TeaserParser::OBJECT_TYPE => new TeaserParser(),
            ImageParser::OBJECT_TYPE => new ImageParser(),
            ImageGalleryParser::OBJECT_TYPE => new ImageGalleryParser(),
            TableParser::OBJECT_TYPE => new TableParser(),
            ContentPartParser::OBJECT_TYPE => new ContentPartParser(new ElementParser()),
            ReviewParser::OBJECT_TYPE => new ReviewParser(new ElementParser()),
            LinkObjectParser::OBJECT_TYPE => new LinkObjectParser(),
            HtmlEmbedParser::OBJECT_TYPE => new HtmlEmbedParser(),
            PdfParser::OBJECT_TYPE => new PdfParser(),
            YoutubeParser::OBJECT_TYPE => new YoutubeParser()
        ];
        $elementParser = new ElementParser();
        $objectParser = new ObjectParser($this->parsers);
        $groupParser = new GroupParser($elementParser, $objectParser);
        $idfParser = new IdfParser($elementParser, $objectParser, $groupParser);

        $this->beConstructedWith($idfParser);
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(NewsMLTransformer::class);
    }
    public function it_can_transform_idf()
    {
        $xml = '
            <idf>
                <group>
                    <element>
                        <span>Tagged</span>
                        <div>Tagless</div>
                        <script>Tagged</script>
                    </element>
                    <object type="x-im/imagegallery" id="imagegallery-1d202026f6d7a8511eeaf522a366d74e">
                        <links>
                            <link rel="image" type="x-im/image"  uri="im://image/NygPCiTpwG3j0WItNRSc3a6AbY0.jpg" uuid="c965e053-6176-58ec-81ad-d2d4c778761c">
                                <data>
                                    <height>1109</height>
                                </data>
                            </link>
                        </links>
                    </object>
                    <element>
                        <span>Tagged</span>
                        <i>Tagged</i>
                        <em>Tagged</em>
                        <b>Tagged</b>
                        <strong>Tagged</strong>
                        <sup>Tagged</sup>
                        <sub>Tagged</sub>
                        <br>Tagged</br>
                        <hr>Tagged</hr>
                        <a>Tagged</a>
                        <script>Tagged</script>
                        <iframe>Tagged</iframe>
                    </element>
                    <object id="youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7" type="x-im/youtube" uri="https://www.youtube.com/watch?v=htqjcfSrQsA" url="https://www.youtube.com/watch?v=htqjcfSrQsA">
                        <data>
                            <start>0</start>
                        </data>
                        <links>
                            <link rel="alternate" title="Click link to view content" type="text/html" url="https://www.youtube.com/watch?v=htqjcfSrQsA">
                                <data>
                                    <context>Video</context>
                                    <provider>YouTube</provider>
                                </data>
                            </link>
                            <link rel="alternate" type="image/jpg" url="https://i.ytimg.com/vi/htqjcfSrQsA/hqdefault.jpg">
                                <data>
                                    <width>480</width>
                                    <height>360</height>
                                </data>
                            </link>
                        </links>
                    </object>
                </group>
            </idf>
            ';
            $this->transform($xml)
                ->shouldHaveArrayKeys([
                    'id' => 'youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7',
                    'type' => 'x-im/youtube',
                    'uri' => 'https://www.youtube.com/embed/htqjcfSrQsA',
                    'url' => 'https://www.youtube.com/watch?v=htqjcfSrQsA',
                    'thumb' => [
                        'width' => '480',
                        'height' => '360',
                        'url' => 'https://i.ytimg.com/vi/htqjcfSrQsA/hqdefault.jpg'
                    ],
                ]);
            $this->transform($xml)
                ->shouldHaveArrayStructureAndValue('<span>Tagged</span>Tagless<script>Tagged</script>', 0);
            $this->transform($xml)
                ->shouldHaveArrayStructureAndValue('<span>Tagged</span><i>Tagged</i><em>Tagged</em><b>Tagged</b><strong>Tagged</strong><sup>Tagged</sup><sub>Tagged</sub><br>Tagged</br><hr>Tagged</hr><a>Tagged</a><script>Tagged</script><iframe>Tagged</iframe>', 2);
        
    }
    public function it_can_parse_an_object()
    {
        $objectParser = new ObjectParser($this->parsers);

        $this->beConstructedWith($objectParser);
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" title="" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                <links>
                    <link rel="self" type="x-im/article" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                    </link>
                </links>
            </object>';
        $this->transform($xml)
            ->shouldHaveArrayIdAndType('54db7504-cb51-4a1a-8c62-137e465e65ba');
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayStructureAndValue' => function ($subject, string $content, int $key ) {
                return $subject[$key]->getItemAttribute('content') === $content ? true : false;
            },
            'haveArrayKeyAndValue' => function ($subject) {
                return $subject[1]->images[0] instanceof ImageItem;
            },
            'haveArrayIdAndType' => function ($subject, string $id) {
                $properties = $subject[0]->getItemAttributes();
                
                return ($properties['uuid'] === $id) && ($subject[0] instanceof Item);
            },
            'haveArrayKeys' => function ($subject, $values) {
                return ($subject[3]->getItemAttributes() == $values) && ($subject[0] instanceof Item);
            },
            'haveArrayIdAndType' => function ($subject, string $id) {
                $properties = $subject[0]->getItemAttributes();
                
                return ($properties['uuid'] === $id) && ($subject[0] instanceof Item);
            },
        ];
    }
}
