<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\ContentPartParser;
use PhpSpec\Exception\Example\FailureException;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use SimpleXMLIterator;
use Infomaker\Everyware\NewsML\Item;


class ContentPartParserSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith(new ElementParser());
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(ContentPartParser::class);
    }
    public function it_must_be_initiated_with_an_ElementParser()
    {
        $this->shouldThrow(\TypeError::class)->during('__construct', [null]);
    }
    public function it_can_parse_an_element()
    {
        $xml = '
            <xml>
                <data>
                    <nil>dsgdsg</nil>
                </data>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)->shouldHaveXmlKeys(['nil', 'text', 'title', 'subject', 'type', 'content_type']);
    }

    public function it_can_return_type_to_be_parsed()
    {
        $this->typeToParse()
            ->shouldReturn('x-im/content-part');
    }
    public function it_should_have_type_set()
    {
        $xml = '
            <xml type="x-im/content-part">
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValue('type', 'x-im/content-part');
    }
    public function it_can_get_content_type()
    {
        $xml = '
            <xml>
                <data>
                    <nil>dsgdsg</nil>
                    <content_type></content_type>
                </data>
                <links><link type="x-im/instagram" url="testat" rel="content-part" uri="im:/content-part/fact">test</link></links>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValue('content_type', 'fact');
    }
    public function it_should_return_defaultData_if_data_is_none_existing()
    {
        $xml = '
            <xml>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
             ->shouldHaveXmlKeys(['title', 'text', 'subject']); 
    }
    public function it_should_parse_text()
    {
        $xml = '
            <xml>
                <data>
                    <text>
                        <string>En</string>
                        <string>massa</string>
                        <string>text</string>
                    </text>
                </data>
                <links><link type="x-im/instagram" url="testat" rel="content-part" uri="im:/content-part/fact">test</link></links>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
            $this->parse($el)->shouldHaveArrayValues(['En', 'massa', 'text']);
    
    }


    public function getMatchers(): array
    {
        return [
            'haveArrayValues' => function ($subject, $valued) {
                $text = $subject[0]->getItemAttribute('text');
                
                foreach ($text as $key => $value) {
                    if($value->getFilteredContent(['']) != $valued[$key])
                        return false;
                }

                return true;
            },
            'haveArrayValue' => function (array $subject, string $key, $value) {
                return array_filter($subject, function($item) use($value, $key) {
                    return ($item->$key === $value);
                });
            },
            'haveXmlKeys' => function (array $subject, array $keys) {
                return array_filter($subject, function($items) use($keys) {
                    if (count(array_intersect_key(array_flip($keys), $items->getItemAttributes())) != count($keys)) {
                        throw new FailureException(
                            sprintf(
                                'the return value "%s" should contain "%s"',
                                get_class($items),
                                implode(', ', $keys)
                            )
                        );
                    }
                    return true;
                });
            },
        ];
    }
}
