<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class ElementParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ElementParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <xml>
                <p>Tagless</p>
                <i>Tagged</i>
                <em>Tagged</em>
                <div>Tagless</div>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveXmlKeys('Tagless<i>Tagged</i><em>Tagged</em>Tagless');
    }
    public function it_should_allow_these_tags()
    {
        $xml = '
            <xml>
                <span>Tagged</span>
                <i>Tagged</i>
                <em>Tagged</em>
                <b>Tagged</b>
                <strong>Tagged</strong>
                <sup>Tagged</sup>
                <sub>Tagged</sub>
                <br>Tagged</br>
                <hr>Tagged</hr>
                <a>Tagged</a>
                <script>Tagged</script>
                <iframe>Tagged</iframe>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveXmlKeys('<span>Tagged</span><i>Tagged</i><em>Tagged</em><b>Tagged</b><strong>Tagged</strong><sup>Tagged</sup><sub>Tagged</sub><br>Tagged</br><hr>Tagged</hr><a>Tagged</a><script>Tagged</script><iframe>Tagged</iframe>');
    }
    public function getMatchers(): array
    {
        return [
            'haveXmlKeys' => function (array $subject, string $content) {
                return $subject[0]->content === $content ? true : false;
            },
        ];
    }
}