<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\GroupParser;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use Infomaker\Everyware\NewsML\Parsers\ImageGalleryParser;
use Infomaker\Everyware\NewsML\Parsers\ObjectParser;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class GroupParserSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith(new ElementParser(), new ObjectParser(['x-im/imagegallery' => new ImageGalleryParser()]));
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(GroupParser::class);
    }
    public function it_can_parse_an_element_with_tags()
    {
        $xml = '
        <group>
            <element>
                <span>Tagged</span>
                <div>Tagless</div>
                <script>Tagged</script>
            </element>
            <object type="x-im/imagegallery" id="imagegallery-1d202026f6d7a8511eeaf522a366d74e">
                <links>
                    <link rel="image" type="x-im/image"  uri="im://image/NygPCiTpwG3j0WItNRSc3a6AbY0.jpg" uuid="c965e053-6176-58ec-81ad-d2d4c778761c">
                        <data>
                            <height>1109</height>
                        </data>
                    </link>
                </links>
            </object>
        </group>';

        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayStructureAndValue('<span>Tagged</span>Tagless<script>Tagged</script>', 0);
        $this->parse($el)   
            ->shouldHaveArrayKeyAndValue();
    }
    public function it_will_return_an_empty_object_if_ObjectParser_initalizes_without_parsers()
    {
        $xml = '
            <group>
                <object type="x-im/imagegallery" id="imagegallery-1d202026f6d7a8511eeaf522a366d74e">
                    <links>
                        <link rel="image" type="x-im/image"  uri="im://image/NygPCiTpwG3j0WItNRSc3a6AbY0.jpg" uuid="c965e053-6176-58ec-81ad-d2d4c778761c">
                            <data>
                                <height>1109</height>
                            </data>
                        </link>
                    </links>
                </object>
            </group>';
        $this->beConstructedWith(new ElementParser(), new ObjectParser([]));
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)   
            ->shouldReturn([]);
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayStructureAndValue' => function ($subject, string $content, int $key ) {
                return $subject[$key]->getItemAttribute('content') === $content ? true : false;
            },
            'haveArrayKeyAndValue' => function ($subject) {
                return $subject[1]->images[0] instanceof ImageItem;
            }
        ];
    }
}
