<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\HtmlEmbedParser;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class HtmlEmbedParserSpec extends ObjectBehavior
{
    private $el, $xml = '
        <xml>
            <data>
                <format>1234</format>
                <text>Content</text>
            </data>
        </xml>
    ';

    public function let() 
    {
        $this->el = simplexml_load_string($this->xml, 'SimpleXMLIterator');
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(HtmlEmbedParser::class);
    }
    public function it_should_parse_an_element()
    {
        $this->parse($this->el)
            ->shouldHaveFormatAndText('Content', '1234');
    }
    public function it_should_return_default_values_if_not_set()
    {
        $xml = '
            <xml>
                <data>
                </data>
            </xml>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveFormatAndText('', 'text');
    }
    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/htmlembed');
    }
    public function getMatchers(): array
    {
        return [
            'haveFormatAndText' => function ($subject, $expectedContent, $expectedFormat) {
                $format = $subject[0]->format;
                $content = $subject[0]->content;
                
                return ($content === $expectedContent) && ($format === $expectedFormat);
            },
        ];
    }
}
