<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\LinkParser;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Infomaker\Everyware\NewsML\Parsers\TwitterLinkParser;
use SimpleXMLIterator;
use Infomaker\Everyware\NewsML\Parsers\InstagramLinkParser;

class LinkParserSpec extends ObjectBehavior
{
    private $parsers;

    public function let()
    {
        $this->parsers = ['x-im/instagram' => new InstagramLinkParser(), 'x-im/tweet' => new TwitterLinkParser()];
        $this->beConstructedWith($this->parsers);
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(LinkParser::class);
    }
    public function it_can_pass_parsers()
    {
        $this->beConstructedWith([]);
        $this->shouldHaveType(LinkParser::class);
    }
    public function it_can_get_a_link_parser()
    {
        $xml = '<link type="x-im/instagram" url="testat">test</link>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->getLinkParser($el)
            ->shouldReturn($this->parsers['x-im/instagram']);
    }
    public function it_can_parse_links()
    {
        $xml = '<xml><links><link type="x-im/instagram" url="testat">test</link></links></xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parseLinks($el)
            ->shouldHaveValue('embed_url', 'https://api.instagram.com/oembed/?url=testat');
    }
    public function it_can_get_a_type()
    {
        $xml = '<link type="x-im/instagram" url="testat">test</link>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->getType($el)
            ->shouldReturn('x-im/instagram');
    }
    public function getMatchers(): array
    {
        return [
            'haveKeys' => function ($subject, $keys) {
                if (count(array_intersect_key(array_flip($keys), $subject)) === count($keys)) {
                    return true;
                }
            },
            'haveValue' => function (array $subject, string $key, $value) {
                return array_filter($subject, function($item) use($value, $key){
                    return ($item->$key === $value);
                 });
            },
        ];
    }
}
