<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\ObjectParser;
use Infomaker\Everyware\NewsML\Parsers\LinkObjectParser;
use Infomaker\Everyware\NewsML\Item;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class ObjectParserSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith(['x-im/link' => new LinkObjectParser()]);
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(ObjectParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" title="" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                <links>
                    <link rel="self" type="x-im/article" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayIdAndType('54db7504-cb51-4a1a-8c62-137e465e65ba');
    }
    public function it_should_return_an_empty_array_if_parsers_are_none_existing()
    {

        $this->beConstructedWith(['x-im/link' => 'nada']);
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" title="" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                <links>
                    <link rel="self" type="x-im/article" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldReturn([]);
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayIdAndType' => function ($subject, string $id) {
                $properties = $subject[0]->getItemAttributes();
                
                return ($properties['uuid'] === $id) && ($subject[0] instanceof Item);
            },
        ];
    }
}
