<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\PdfParser;
use Infomaker\Everyware\NewsML\Item;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class PdfParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(PdfParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                <links>
                    <link rel="self" type="x-im/article" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba" uri="im://pdf/slug" title="pdfTitle">
                        <data>
                            <text>pdfTitle</text>
                        </data>
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeys([
                'type' => 'x-im/link',
                'id' => 'contentrelations-c0580e5c1a9fac463b6989b2bc48fb10',
                'uuid' => '54db7504-cb51-4a1a-8c62-137e465e65ba',
                'title' => 'pdfTitle',
                'file' => 'slug',
                'url' => 'randomPdf/slug'
            ]);
    }
    public function it_should_parse_an_element_even_if_its_empty()
    {
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
              
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeys([
                'type' => 'x-im/link',
                'id' => 'contentrelations-c0580e5c1a9fac463b6989b2bc48fb10',
                'uuid' => '54db7504-cb51-4a1a-8c62-137e465e65ba',
            ]);
    }
    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/pdf');
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayValuesAndKeys' => function ($subject, $values) {
                return ($subject[0]->getItemAttributes() === $values) && ($subject[0] instanceof Item);
            },
        ];
    }
}
