<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\ReviewParser;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use PhpSpec\ObjectBehavior;
use PhpSpec\Exception\Example\FailureException;
use Prophecy\Argument;

class ReviewParserSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith(new ElementParser());
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(ReviewParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <xml>
                <data>
                    <text>
                        <string>En</string>
                        <string>Massa</string>
                        <string>Text?</string>
                    </text>
                </data>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValues(['En', 'Massa', 'Text?']);
    }
    public function it_should_return_defaultData_when_parsed_without_data()
    {
        $xml = '
            <xml>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveXmlKeys(['score', 'text']);
    }
    public function it_should_return_score_and_text_with_set_values()
    {
        $xml = '
            <xml>
                <data>
                    <text>
                        <string>En</string>
                        <string>Massa</string>
                        <string>Text?</string>
                    </text>
                    <score>1337</score>
                </data>
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValue('1337');
        $this->parse($el)
            ->shouldHaveArrayValues(['En', 'Massa', 'Text?']);
    }
    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-gm/review');
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayValues' => function ($subject, $keys) {
                $text = $subject[0]->getItemAttribute('text');
                foreach ($text as $key => $value) {
                    if($value->getFilteredContent(['']) != $keys[$key])
                        return false;
                }

                return true;
            },
            'haveXmlKeys' => function (array $subject, array $keys) {
                return array_filter($subject, function($items) use($keys) {
                    if (count(array_intersect_key(array_flip($keys), $items->getItemAttributes())) != count($keys)) {
                        throw new FailureException(
                            sprintf(
                                'the return value "%s" should contain "%s"',
                                get_class($items),
                                implode(', ', $keys)
                            )
                        );
                    }
                    return true;
                });
            },
            'haveArrayValue' => function ($subject, $value) {
                return $subject[0]->getItemAttribute('score') === $value ?: false;
            },
        ];
    }
}
